@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2014 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #ifndef __SIEVE_INTERPRETER_H
@@ -16,14 +16,22 @@
  */
 
 struct sieve_interpreter *sieve_interpreter_create
-	(struct sieve_binary *sbin, const struct sieve_message_data *msgdata,
-		const struct sieve_script_env *senv, struct sieve_error_handler *ehandler,
-		enum sieve_runtime_flags flags);
-struct sieve_interpreter *sieve_interpreter_create_for_block
-	(struct sieve_binary_block *sblock, struct sieve_script *script,
+	(struct sieve_binary *sbin,
+		struct sieve_interpreter *parent,
 		const struct sieve_message_data *msgdata,
-		const struct sieve_script_env *senv, struct sieve_error_handler *ehandler,
-		enum sieve_runtime_flags flags);
+		const struct sieve_script_env *senv,
+		struct sieve_error_handler *ehandler,
+		enum sieve_execute_flags flags)
+	ATTR_NULL(2);
+struct sieve_interpreter *sieve_interpreter_create_for_block
+	(struct sieve_binary_block *sblock,
+		struct sieve_script *script,
+		struct sieve_interpreter *parent,
+		const struct sieve_message_data *msgdata,
+		const struct sieve_script_env *senv,
+		struct sieve_error_handler *ehandler,
+		enum sieve_execute_flags flags);
+	ATTR_NULL(3);
 void sieve_interpreter_free(struct sieve_interpreter **_interp);
 
 /*
@@ -32,6 +40,8 @@ void sieve_interpreter_free(struct sieve_interpreter **_interp);
 
 pool_t sieve_interpreter_pool
 	(struct sieve_interpreter *interp);
+struct sieve_interpreter *
+sieve_interpreter_get_parent(struct sieve_interpreter *interp);
 struct sieve_script *sieve_interpreter_script
 	(struct sieve_interpreter *interp);
 struct sieve_error_handler *sieve_interpreter_get_error_handler
@@ -45,6 +55,43 @@ struct sieve_instance *sieve_interpreter_svinst
 void sieve_interpreter_set_result
 	(struct sieve_interpreter *interp, struct sieve_result *result);
 
+/*
+ * Loop handling
+ */
+
+struct sieve_interpreter_loop;
+
+int sieve_interpreter_loop_start
+	(struct sieve_interpreter *interp, sieve_size_t loop_end,
+		const struct sieve_extension_def *ext_def,
+		struct sieve_interpreter_loop **loop_r);
+struct sieve_interpreter_loop *sieve_interpreter_loop_get
+	(struct sieve_interpreter *interp, sieve_size_t loop_end,
+		const struct sieve_extension_def *ext_def);
+int sieve_interpreter_loop_next
+	(struct sieve_interpreter *interp,
+		struct sieve_interpreter_loop *loop,
+		sieve_size_t loop_begin);
+int sieve_interpreter_loop_break
+	(struct sieve_interpreter *interp,
+		struct sieve_interpreter_loop *loop);
+
+struct sieve_interpreter_loop *sieve_interpreter_loop_get_local
+(struct sieve_interpreter *interp,
+	struct sieve_interpreter_loop *loop,
+	const struct sieve_extension_def *ext_def) ATTR_NULL(2, 3);
+struct sieve_interpreter_loop *sieve_interpreter_loop_get_global
+(struct sieve_interpreter *interp,
+	struct sieve_interpreter_loop *loop,
+	const struct sieve_extension_def *ext_def) ATTR_NULL(2, 3);
+
+pool_t sieve_interpreter_loop_get_pool
+	(struct sieve_interpreter_loop *loop) ATTR_PURE;
+void *sieve_interpreter_loop_get_context
+	(struct sieve_interpreter_loop *loop) ATTR_PURE;
+void sieve_interpreter_loop_set_context
+	(struct sieve_interpreter_loop *loop, void *context);
+
 /*
  * Program flow
  */
@@ -57,7 +104,7 @@ sieve_size_t sieve_interpreter_program_counter
 	(struct sieve_interpreter *interp);
 
 int sieve_interpreter_program_jump
-	(struct sieve_interpreter *interp, bool jump);
+	(struct sieve_interpreter *interp, bool jump, bool break_loops);
 
 /*
  * Test results
@@ -107,12 +154,13 @@ int sieve_runtime_mail_error
 struct sieve_interpreter_extension {
 	const struct sieve_extension_def *ext_def;
 
-	void (*run)
-		(const struct sieve_extension *ext, const struct sieve_runtime_env *renv,
-			void *context);
+	int (*run)
+		(const struct sieve_extension *ext,
+			const struct sieve_runtime_env *renv,
+			void *context, bool deferred);
 	void (*free)
-		(const struct sieve_extension *ext, struct sieve_interpreter *interp,
-			void *context);
+		(const struct sieve_extension *ext,
+			struct sieve_interpreter *interp, void *context);
 };
 
 void sieve_interpreter_extension_register
@@ -124,6 +172,10 @@ void sieve_interpreter_extension_set_context
 void *sieve_interpreter_extension_get_context
 	(struct sieve_interpreter *interp, const struct sieve_extension *ext);
 
+int sieve_interpreter_extension_start
+	(struct sieve_interpreter *interp,
+		const struct sieve_extension *ext);
+
 /*
  * Opcodes and operands
  */
